(function () {
    "use strict";

    var TranslationApiLink = "https://shamra.sy/translate/api/translate";

    //Execute the logic of Translation bundle
    displayTranslator();

    var config = {};

    /**
     * The logic of the Exchange Bundle
     * @returns {undefined}
     */
    function displayTranslator() {
        //Get the current script so we can fetch client data
        var currentScript = getCurrentScript();
        /**
         * Get the client configuration:
         *      the token of the client in the attribute 'data-token'
         *      the widget id in the attribute 'data-target'
         * @type {{token: string|null, target: (string)}}
         */
        config = {
            token: currentScript.getAttribute('data-token'),
            target: currentScript.getAttribute('data-target') || 'shamra-translation-widget',
            type: currentScript.getAttribute('data-type') || 'small',
            options: currentScript.getAttribute('data-options') || '{}'
        };
        //Get all validation errors and print them inside the console
        var validationResult = validateConfiguration();
        if (validationResult.length > 0) {
            for (var i = 0; i < validationResult.length; i++) {
                console.error(validationResult[i]);
            }
            displayErrorBeforeLoad(config.target);
            return undefined;
        }
        config.token = config.token.trim();
        config.target = config.target.trim();
        config.type = config.type.trim().toLowerCase();
        config.options = config.options.trim().toLowerCase();
        //Requesting data from Shamra Translation API
        requestData();
    }


    /**
     * Function that gets the current script tag at the clients page so that we can fetch data attributes
     * @return Current script element
     */
    function getCurrentScript() {
        /**
         * The document.currentScript is supported in all browsers except IE,
         * so if the client is using IE, we fetch it using the logic in the getCurrentScriptForIE() function
         */
        if (!document.currentScript) {
            document.currentScript = getCurrentScriptForIE();
        }
        return document.currentScript;
    }

    /**
     *
     * @return {Node}
     */
    function getCurrentScriptForIE() {
        //Get all scripts loaded for this moment
        var scripts = document.getElementsByTagName('script');
        //Return the last loaded one, which is this script
        return scripts[scripts.length - 1];
    }

    /**
     * Function validates that the token and the target element id are set
     *
     * @returns {Array}
     */
    function validateConfiguration() {
        //Errors array to be returned
        var errors = [];
        //If the token is null or empty
        if (!(config.token)) {
            errors.push('Your token is empty or not set, please check our documentation!');
        }
        //If the target is null or empty
        if (!(config.target)) {
            errors.push('Your target element is empty or not set, please check our documentation!');
        }
        //If the target is null or empty
        if (!(config.type)) {
            errors.push('Your didn\'t select a valid type, please check our documentation!');
        }else{
            if(!(config.type.toLowerCase() === 'large' || config.type.toLowerCase() === 'small')){
                errors.push('Your didn\'t select a valid type, type is either "large" or "small", please check our documentation!');
            }
        }
        if(!!(config.options)){
            try{
                var options = JSON.parse(config.options);
            }catch (ex){
                errors.push('Options should be either empty, or written in JSON format!');
            }
        }
        return errors;
    }

    /**
     * Function that requests exchange data from Shamra Translation API
     */
    function requestData() {
        var xmlhttp;
        if (window.XMLHttpRequest) {
            //This code is for all browsers except IE
            xmlhttp = new XMLHttpRequest();
        } else {
            //This code is for IE
            xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
        }
        var ApiLink = TranslationApiLink;
        var targetId = config.target;
        var token = config.token;
        var type = config.type;
        var options = JSON.parse("{}");
        if(!!(config.options)){
            options = JSON.parse(config.options);
        }
        xmlhttp.onreadystatechange = function () {
            if(this.readyState === 4) {
                if (this.status === 200) {
                    receiveTranslationData(this.responseText, targetId , type.toLowerCase(), token, options);
                } else if (this.status === 401) {
                    console.error('It seems like there was an error with your authentication, please check that your token { ' + token + ' } is correct!');
                    displayErrorBeforeDetectingTarget(targetId);
                } else if (this.status === 405 || this.status === 500) {
                    console.error('Sorry! we are having some defficulties!');
                    displayErrorBeforeDetectingTarget(targetId);
                } else {
                    console.error('An error occured, please check your configuration at Shamra Developer, status code {' + this.status + '}');
                    displayErrorBeforeDetectingTarget(targetId);
                }
            }else if(this.readyState === 0){
                console.error('An error occured during the initiating of the request!');
                displayErrorBeforeDetectingTarget(targetId);
            }
        };
        xmlhttp.open("POST", ApiLink + '?token=' + token + '&validation=true' , true);
        xmlhttp.setRequestHeader('Accept' , 'application/json');
        try {
            if(window.addEventListener){
                window.addEventListener('load', function(){xmlhttp.send();})
            }else{
                window.attachEvent('onload', function(){xmlhttp.send();})
            }
        }catch(e) {
            displayErrorBeforeLoad(targetId);
        }
    }

    /**
     * Format the received data from the api and prepare it for display
     * @param requestResult
     * @param targetId
     * @param targetType
     * @param token
     * @param options
     */
    function receiveTranslationData(requestResult , targetId , targetType , token , options) {
        //the following three lines are for testing purposes
        var testingElement = document.getElementById('received');
        if(!!testingElement){
            testingElement.innerText = requestResult;
        }
        //If the target element wasn't found, notify the user
        var targetElement = document.getElementById(targetId);
        if(!!targetElement){
            targetElement.style.direction = 'rtl';
            targetElement.style.textAlign = 'center';
            //targetElement.innerHTML = requestResult;
            //Checking if the received data has a JSON format
            try {
                displayReceivedData(targetElement, targetType, token, options)
            }catch (e){
                console.error('The received data seems to bee corrupted, try again later!');
                displayError(targetElement);
            }
        }else{
            console.error('Shamra Translator API can\'t find the element with the id { ' + targetId + ' }');
        }
    }

    /**
     * Display the translation widget in the selected div
     *
     * @param targetElement
     * @param targetType
     * @param token
     * @param options
     */
    function displayReceivedData( targetElement, targetType, token, options) {
        ceateStyleTag(targetType, options);
        targetElement.innerHTML = '';
        targetElement.style.fontFamily = "'Cairo', sans-serif";
        var translationWidget = getTranslationWidget(targetType, token, options);
        targetElement.appendChild(translationWidget);
    }

    /**
     * Get the translation widget according to the chosen type
     *
     * @param targetType
     * @param token
     * @param options
     */
    function getTranslationWidget(targetType , token, options){

        var rowsOption = getRowsOption(options);

        var widgetDiv = document.createElement('div');
        widgetDiv.classList.add('shamra-translation-widget');
        var transWidgetHeader = document.createElement('div');
        transWidgetHeader.id = 'transWidgetHeader';
        transWidgetHeader.onclick = function () {
            var win = window.open('https://shamra.sy/translate?', '_blank');
            win.focus();
        };
        var translationLogo = document.createElement('img');
        translationLogo.src = 'https://shamra.sy/bundles/syndextranslation/images/CommunityLOGO.png';
        var headerSpan = document.createElement('span');
        headerSpan.appendChild(document.createTextNode('الترجمة من شمرا'));
        transWidgetHeader.appendChild(translationLogo);
        transWidgetHeader.appendChild(headerSpan);

        widgetDiv.appendChild(transWidgetHeader);

        if(targetType === 'small'){
            /**
             *  Create Header of switching between languages
             */
            var transWidgetSecondHeader = document.createElement('div');
            transWidgetSecondHeader.id = 'transWidgetSecondHeader';
            var transSrcAr = document.createElement('div');
            transSrcAr.id = 'trans-src-ar';
            transSrcAr.classList.add('active');
            transSrcAr.onclick = function () {
                shamraTranslationWidgetDoSwitchLangSmall('ar');
            };
            transSrcAr.appendChild(document.createTextNode('عربي'));
            var transSrcEn = document.createElement('div');
            transSrcEn.id = 'trans-src-en';
            transSrcEn.onclick = function () {
                shamraTranslationWidgetDoSwitchLangSmall('en');
            };
            transSrcEn.appendChild(document.createTextNode('English'));
            transWidgetSecondHeader.appendChild(transSrcAr);
            transWidgetSecondHeader.appendChild(transSrcEn);

            widgetDiv.appendChild(transWidgetSecondHeader);

            /**
             * Create translation widget body
             */
            var transWidgetBody = document.createElement('div');
            transWidgetBody.id = 'transWidgetBody';
            transWidgetBody.setAttribute('data-src' , 'ar');
            transWidgetBody.setAttribute('data-tgt' , 'en');
            transWidgetBody.setAttribute('data-token' , token);

            /**
             * Create Source part of the widget body
             */
            var transSrc = document.createElement('div');
            transSrc.id = 'trans-src';
            var transSrcInput = document.createElement('div');
            transSrcInput.id = 'trans-src-input';
            var transSrcInputText = document.createElement('textarea');
            transSrcInputText.id = 'trans-src-input-text';
            transSrcInputText.placeholder = 'أدخل النص بالعربية...';
            transSrcInputText.style.textAlign = 'right';
            transSrcInputText.style.direction = 'rtl';
            transSrcInputText.setAttribute('rows' , rowsOption);
            transSrcInput.appendChild(transSrcInputText);
            transSrc.appendChild(transSrcInput);
            transWidgetBody.appendChild(transSrc);

            /**
             * Create Target part of the widget body
             */
            var transTgt = document.createElement('div');
            transTgt.id = 'trans-tgt';
            var transTgtInput = document.createElement('div');
            transTgtInput.id = 'trans-tgt-input';
            var transTgtInputText = document.createElement('textarea');
            transTgtInputText.setAttribute('readonly','');
            transTgtInputText.id = 'trans-tgt-input-text';
            transTgtInputText.placeholder = 'Translation...';
            transTgtInputText.style.textAlign = 'left';
            transTgtInputText.style.direction = 'ltr';
            transTgtInputText.setAttribute('rows' , rowsOption);
            transTgtInput.appendChild(transTgtInputText);
            transTgt.appendChild(transTgtInput);
            transWidgetBody.appendChild(transTgt);

            widgetDiv.appendChild(transWidgetBody);

            /**
             * Create Tranlsation Button
             */

            var transWidgetButton = document.createElement('div');
            transWidgetButton.id = 'transWidgetButton';
            var translationLink = document.createElement('a');
            translationLink.href = '#';
            translationLink.onclick = function () {
                shamraTranslationWidgetDoTranslate();
            };
            translationLink.appendChild(document.createTextNode('ترجم!'));
            transWidgetButton.appendChild(translationLink);

            widgetDiv.appendChild(transWidgetButton);
        }else{
            /**
             * Create translation widget body
             */
            var transWidgetBody = document.createElement('div');
            transWidgetBody.id = 'transWidgetBody';
            transWidgetBody.setAttribute('data-src' , 'ar');
            transWidgetBody.setAttribute('data-tgt' , 'en');
            transWidgetBody.setAttribute('data-token' , token);

            /**
             * Create Source part of the widget body
             */
            var transSrc = document.createElement('div');
            transSrc.id = 'trans-src';
            transSrc.classList.add('trans-float-left');
            var transSrcTitle = document.createElement('div');
            transSrcTitle.id = 'trans-src-title';
            var transSrcTitleText = document.createElement('span');
            transSrcTitleText.id = 'trans-src-title-text';
            transSrcTitleText.appendChild(document.createTextNode('عربي'))
            transSrcTitle.appendChild(transSrcTitleText);
            var transSwitcher = document.createElement('div');
            transSwitcher.classList.add('trans-switcher');
            transSrc.appendChild(transSrcTitle);

            var transSrcInput = document.createElement('div');
            transSrcInput.id = 'trans-src-input';
            var transSrcInputText = document.createElement('textarea');
            transSrcInputText.id = 'trans-src-input-text';
            transSrcInputText.placeholder = 'أدخل النص بالعربية...';
            transSrcInputText.style.textAlign = 'right';
            transSrcInputText.style.direction = 'rtl';
            transSrcInputText.setAttribute('rows' , rowsOption);
            transSrcInput.appendChild(transSrcInputText);
            transSrc.appendChild(transSrcTitle);
            transSrc.appendChild(transSrcInput);
            transWidgetBody.appendChild(transSrc);

            /**
             * Create Target part of the widget body
             */
            var transTgt = document.createElement('div');
            transTgt.id = 'trans-tgt';
            transTgt.classList.add('trans-float-left');
            var transTgtTitle = document.createElement('div');
            transTgtTitle.id = 'trans-tgt-title';
            var transTgtTitleText = document.createElement('span');
            transTgtTitleText.id = 'trans-tgt-title-text';
            transTgtTitleText.appendChild(document.createTextNode('English'))
            transTgtTitle.appendChild(transTgtTitleText);
            transTgt.appendChild(transTgtTitle);
            var transTgtInput = document.createElement('div');
            transTgtInput.id = 'trans-tgt-input';
            var transTgtInputText = document.createElement('textarea');
            transTgtInputText.setAttribute('readonly','');
            transTgtInputText.id = 'trans-tgt-input-text';
            transTgtInputText.placeholder = 'Translation...';
            transTgtInputText.style.textAlign = 'left';
            transTgtInputText.style.direction = 'ltr';
            transTgtInputText.setAttribute('rows' , rowsOption);
            transTgtInput.appendChild(transTgtInputText);
            transTgt.appendChild(transTgtInput);
            transWidgetBody.appendChild(transTgt);

            /**
             * Create Clerer
             */
            var clearer = document.createElement('div');
            clearer.classList.add('clearer');
            transWidgetBody.appendChild(clearer);

            widgetDiv.appendChild(transWidgetBody);


            var bottomButtons = document.createElement('div');
            bottomButtons.style.textAlign = 'right';
            bottomButtons.style.marginTop = '7px';

            var translationLink = document.createElement('a');
            translationLink.href = '#';
            translationLink.id = 'shamra-translation-button-do-translate';
            translationLink.appendChild(document.createTextNode('ترجمة'));
            translationLink.onclick = function (e) {
                e.preventDefault();
                shamraTranslationWidgetDoTranslate();
            };
            bottomButtons.appendChild(translationLink);

            var swichLangLink = document.createElement('a');
            swichLangLink.href = '#';
            swichLangLink.id = 'shamra-translation-button-switch-language';
            swichLangLink.appendChild(document.createTextNode('تبديل اللغة'));
            swichLangLink.onclick = function (e) {
                e.preventDefault();
                shamraTranslationWidgetDoSwitchLangLarge();
            };
            bottomButtons.appendChild(swichLangLink);

            widgetDiv.appendChild(bottomButtons);
        }

        return widgetDiv;
    }

    function getRowsOption(options) {
        var rowsOption = '3';
        if(options.hasOwnProperty('rows')){
            if(!!(options.rows)){
                rowsOption = options.rows;
            }
        }
        return rowsOption;
    }

    function getFontSizeOption(options) {
        var fontSizeOption = '13px';
        if(options.hasOwnProperty('font_size')){
            if(!!(options.font_size)){
                fontSizeOption = options.font_size;
            }
        }
        return fontSizeOption;
    }

    /**
     * Create style tag to style the widget
     */
    function ceateStyleTag(targetType, options) {
        var head = document.head || document.getElementsByTagName('head')[0];
        var fontCario = document.createElement('link');
        fontCario.href = "https://fonts.googleapis.com/css?family=Cairo";
        fontCario.rel = "stylesheet";
        head.appendChild(fontCario);
        var style = document.createElement('style');
        style.type = 'text/css';
        var css = getCss(targetType, options);
        style.appendChild(document.createTextNode(css));
        head.appendChild(style);
    }

    function getCss(targetType, options) {
        var fontSizeOption = getFontSizeOption(options);
        var css = '';
        if(targetType === 'small'){
            css = '' +
                '.shamra-translation-widget{' +
                '   direction: rtl;' +
                '   font-family: \'Cairo\', sans-serif;' +
                '}' +
                '.shamra-translation-widget #transWidgetHeader{' +
                '   text-align: right;' +
                '   color: #00A550;' +
                '   font-size: 15px;' +
                '   margin-bottom: 5px;' +
                '   cursor: pointer;' +
                '}' +
                '.shamra-translation-widget #transWidgetHeader img{' +
                '   height: 25px;' +
                '   vertical-align: middle;' +
                '   margin-left: 10px;' +
                '}' +
                '.shamra-translation-widget #transWidgetSecondHeader{' +
                '   display: flex;' +
                '   align-items: center;' +
                '   justify-content: center;' +
                '   border-radius: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetSecondHeader #trans-src-ar,' +
                '.shamra-translation-widget #transWidgetSecondHeader #trans-src-en{' +
                '   text-align: center;' +
                '   width: 50%;' +
                '   margin: 0;' +
                '   border: 1px solid #ccc;' +
                '   font-size: ' + fontSizeOption + ";" +
                '   margin-bottom: 5px;' +
                '   cursor: pointer;' +
                '   transition: .2s;' +
                '}' +
                '.shamra-translation-widget #transWidgetSecondHeader #trans-src-ar:hover,' +
                '.shamra-translation-widget #transWidgetSecondHeader #trans-src-en:hover{' +
                '   background-color: #d4d4d4;' +
                '}' +
                '.shamra-translation-widget #transWidgetSecondHeader #trans-src-ar.active,' +
                '   .shamra-translation-widget #transWidgetSecondHeader #trans-src-en.active{' +
                '   background-color: #e6e6e6;' +
                '   box-shadow: inset 0 3px 5px rgba(0,0,0,.125);' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt{' +
                '   margin: 0;' +
                '   border: 1px solid #00a550;' +
                '   padding: 5px;' +
                '   font-size: 12px;' +
                '   color: #757474;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src{' +
                '   border-top-right-radius: 5px;' +
                '   border-top-left-radius: 5px;' +
                '   margin-bottom: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt{' +
                '   border-bottom-right-radius: 5px;' +
                '   border-bottom-left-radius: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-input textarea,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt #trans-tgt-input textarea{' +
                '   font-family: \'Cairo\', sans-serif;' +
                '   font-size: ' + fontSizeOption + ";" +
                '   border: 0;' +
                '   width: 100%;' +
                '   resize: none;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-input textarea:focus,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt #trans-tgt-input textarea:focus{' +
                '   border: 0;' +
                '   outline: none;' +
                '}' +
                '.shamra-translation-widget #transWidgetButton{' +
                '   text-align: center;' +
                '   margin-top: 10px;' +
                '}' +
                '.shamra-translation-widget #transWidgetButton a{' +
                '   font-size: ' + fontSizeOption + ";" +
                '   color: #00a550;' +
                '   background-color: #fff;' +
                '   border: 1px solid #00a550;' +
                '   padding: 1px;' +
                '   display: block;' +
                '   text-decoration: none;' +
                '   transition: .2s;' +
                '}' +
                '.shamra-translation-widget #transWidgetButton a:hover{' +
                '   color: #fff;' +
                '   background-color: #00a550;' +
                '}';
        }else{
            css = '' +
                '.shamra-translation-widget{' +
                '   direction: rtl;' +
                '   font-family: \'Cairo\', sans-serif;' +
                '}' +
                '.shamra-translation-widget #transWidgetHeader{' +
                '   text-align: right;' +
                '   color: #00A550;' +
                '   font-size: 15px;' +
                '   margin-bottom: 10px;' +
                '   cursor: pointer;' +
                '}' +
                '.shamra-translation-widget #transWidgetHeader img{' +
                '   height: 25px;' +
                '   vertical-align: middle;' +
                '   margin-left: 10px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody{' +
                '   display: flex;' +
                '   align-items: center;' +
                '   justify-content: center;' +
                '   box-shadow: 0 2px 2px #ccc;' +
                '   border-radius: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody .trans-float-left{' +
                '   float: right;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt{' +
                '   width: 50%;' +
                '   margin: 0;' +
                '   border: 1px solid #00a550;' +
                '   padding: 5px;' +
                '   font-size: ' + fontSizeOption + ";" +
                '   color: #757474;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src{' +
                '   border-left: 0;' +
                '   border-top-right-radius: 5px;' +
                '   border-bottom-right-radius: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt{' +
                '   border-top-left-radius: 5px;' +
                '   border-bottom-left-radius: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-title,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt #trans-tgt-title{' +
                '   text-align: center;' +
                '   border-bottom: 1px solid #00a550;' +
                '   padding-bottom: 5px;' +
                '   margin-bottom: 5px;' +
                '   position: relative;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-title .trans-switcher{' +
                '   position: absolute;' +
                '   top:0;' +
                '   left: 5px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-title img{' +
                '   width: 23px;' +
                '   height: 23px;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-input textarea,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt #trans-tgt-input textarea{' +
                '   font-family: \'Cairo\', sans-serif;' +
                '   font-size: ' + fontSizeOption + ";" +
                '   border: 0;' +
                '   width: 100%;' +
                '   resize: none;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody #trans-src #trans-src-input textarea:focus,' +
                '.shamra-translation-widget #transWidgetBody #trans-tgt #trans-tgt-input textarea:focus{' +
                '   border: 0;' +
                '   outline: none;' +
                '}' +
                '.shamra-translation-widget #transWidgetBody .clearer{' +
                '   clear: both;' +
                '}' +
                '.shamra-translation-widget #shamra-translation-button-do-translate{' +
                '   font-family: \'Cairo\', sans-serif;' +
                '   font-size: ' + fontSizeOption + ";" +
                '   border: 1px solid #00a550;' +
                '   border-radius: 4px;' +
                '   padding: 1px;' +
                '   padding-left: 15px;' +
                '   padding-right: 15px;' +
                '   text-decoration: none;' +
                '   transition: .2s;' +
                '   color: #fff;' +
                '   background-color: #00a550;' +
                '}' +
                '' +
                '.shamra-translation-widget #shamra-translation-button-do-translate:hover{' +
                '   color: #00a550;' +
                '   background-color: #fff;' +
                '}'+
                '.shamra-translation-widget #shamra-translation-button-switch-language{' +
                '   font-family: \'Cairo\', sans-serif;' +
                '   font-size: ' + fontSizeOption + ";" +
                '   margin-right: 5px;' +
                '   border: 1px solid #ccc;' +
                '   border-radius: 4px;' +
                '   padding: 1px;' +
                '   padding-left: 15px;' +
                '   padding-right: 15px;' +
                '   text-decoration: none;' +
                '   transition: .2s;' +
                '   color: #000;' +
                '   background-color: #fff;' +
                '}' +
                '' +
                '.shamra-translation-widget #shamra-translation-button-switch-language:hover{' +
                '   background-color: #e6e6e6;' +
                '}';
        }
        return css;
    }

    function displayErrorBeforeLoad(targetId) {
        window.onload = function () {
            var targetElement = document.getElementById(targetId);
            if(!!targetElement){
                displayError(targetElement);
            }
        }
    }

    function displayErrorBeforeDetectingTarget(targetId) {
        var targetElement = document.getElementById(targetId);
        if(!!targetElement){
            displayError(targetElement);
        }
    }

    function displayError(targetElement) {
        var errorDiv = document.createElement('div');
        errorDiv.id = 'exchange-api-error';
        errorDiv.style.backgroundColor = 'whitesmoke';
        errorDiv.style.padding = '10px';
        errorDiv.style.direction = 'rtl';
        errorDiv.style.textAlign = 'center';
        errorDiv.appendChild(document.createTextNode('شمرا ترجمة . حدث خطأ أثناء التنفيذ، تحقق من منطقة عرض الأخطاء الخاصة بمتصفحك!'));
        targetElement.appendChild(errorDiv);
    }

})();

function textIsNullOrEmpty(value) {
    return (!value || value == undefined || value == "" || value.length == 0);
}

function shamraTranslationWidgetDoTranslate() {
    var transWidgetBody = document.getElementById('transWidgetBody');
    var dataSrc = transWidgetBody.getAttribute('data-src');
    var dataTgt = transWidgetBody.getAttribute('data-tgt');
    var dataToken = transWidgetBody.getAttribute('data-token');
    var srcInput = document.getElementById('trans-src-input-text');
    dataText = srcInput.value;
    if(!textIsNullOrEmpty(dataText)) {
        var xmlhttp;
        if (window.XMLHttpRequest) {
            //This code is for all browsers except IE
            xmlhttp = new XMLHttpRequest();
        } else {
            //This code is for IE
            xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
        }
        //var ApiLink = "http://783-add-statistics-for-developer-api-services-and-applications.myshamra.com/translate/api/translate";
        var ApiLink = "https://shamra.sy/translate/api/translate";

        xmlhttp.onreadystatechange = function () {
            if (this.readyState === 4) {
                if (this.status == 200) {
                    var tgt = document.getElementById('trans-tgt-input-text');
                    var result = JSON.parse(this.responseText);
                    tgt.value = result.translation;
                }else{
                    var tgt = document.getElementById('trans-tgt-input-text');
                    var srcLang = transWidgetBody.getAttribute('data-src');
                    if(srcLang == 'ar'){
                        tgt.value = 'An unexpected error occured';
                    }else{
                        tgt.value = 'حدث خطأ غير متوقع.';
                    }
                }
            } else if (this.readyState === 0) {
                console.error('An error occured during the initiating of the request!');
                //displayErrorBeforeDetectingTarget(targetId);
            }
        };
        xmlhttp.open("POST", ApiLink + '?token=' + dataToken, true);
        xmlhttp.setRequestHeader('Accept', 'application/json');
        xmlhttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded; charset=UTF-8');
        try {
            var tgtInput = document.getElementById('trans-tgt-input-text');
            tgtInput.value = '';
            var sentData = 'source_language=' + dataSrc + '&target_language=' + dataTgt + '&text=' + encodeURIComponent(dataText);
            xmlhttp.send(sentData);
        } catch (e) {
            alert('An unknown error occurred!');
            //displayErrorBeforeLoad(targetId);
        }
    }
}

function shamraTranslationWidgetDoSwitchLangLarge() {
    var transWidgetBody = document.getElementById('transWidgetBody');
    var dataSrc = transWidgetBody.getAttribute('data-src');
    var dataTgt = transWidgetBody.getAttribute('data-tgt');
    transWidgetBody.setAttribute('data-src' , dataTgt);
    transWidgetBody.setAttribute('data-tgt' , dataSrc);
    var transSrcTitleText = document.getElementById('trans-src-title-text');
    var transTgtTitleText = document.getElementById('trans-tgt-title-text');
    var temp = transSrcTitleText.innerText;
    transSrcTitleText.innerText = transTgtTitleText.innerText;
    transTgtTitleText.innerText = temp;
    var tgtInput = document.getElementById('trans-tgt-input-text');
    var srcInput = document.getElementById('trans-src-input-text');
    srcInput.value = tgtInput.value;
    tgtInput.value = '';
    if(dataSrc == 'ar'){
        srcInput.style.direction = 'ltr';
        srcInput.style.textAlign = 'left';
        srcInput.placeholder = 'Enter text in english...';
        tgtInput.style.direction = 'rtl';
        tgtInput.style.textAlign = 'right';
        tgtInput.placeholder = 'الترجمة...';
    }else{
        srcInput.style.direction = 'rtl';
        srcInput.style.textAlign = 'right';
        srcInput.placeholder = 'أدخل النص بالعربية...';
        tgtInput.style.direction = 'ltr';
        tgtInput.style.textAlign = 'left';
        tgtInput.placeholder = 'translation...';
    }
    shamraTranslationWidgetDoTranslate();
}

function shamraTranslationWidgetDoSwitchLangSmall(lang){
    var transWidgetBody = document.getElementById('transWidgetBody');
    var dataSrc = transWidgetBody.getAttribute('data-src');
    var dataTgt = transWidgetBody.getAttribute('data-tgt');
    if(!((lang === 'ar' && dataSrc === 'ar') || (lang === 'en' && dataSrc === 'en'))){
        var tgtInput = document.getElementById('trans-tgt-input-text');
        var srcInput = document.getElementById('trans-src-input-text');
        srcInput.value = tgtInput.value;
        tgtInput.value = '';
        transWidgetBody.setAttribute('data-src' , dataTgt);
        transWidgetBody.setAttribute('data-tgt' , dataSrc);
        var btnAr = document.getElementById('trans-src-ar');
        var btnEn = document.getElementById('trans-src-en');
        if(dataSrc == 'ar'){
            btnAr.classList.remove('active');
            btnEn.classList.add('active');
            srcInput.style.direction = 'ltr';
            srcInput.style.textAlign = 'left';
            srcInput.placeholder = 'Enter text in english...';
            tgtInput.style.direction = 'rtl';
            tgtInput.style.textAlign = 'right';
            tgtInput.placeholder = 'الترجمة...';
        }else{
            btnAr.classList.add('active');
            btnEn.classList.remove('active');
            srcInput.style.direction = 'rtl';
            srcInput.style.textAlign = 'right';
            srcInput.placeholder = 'أدخل النص بالعربية...';
            tgtInput.style.direction = 'ltr';
            tgtInput.style.textAlign = 'left';
            tgtInput.placeholder = 'translation...';
        }
        shamraTranslationWidgetDoTranslate();
    }
}